from nelsnmp.hostinfo.collect import get_device_version
from nelsnmp.oids import GeneralOids
from nelsnmp.snmp import SnmpHandler
from nelsnmp.vendors.mappings import vendor_map
o = GeneralOids()

class HostInfo(object):

    def __init__(self, Snmp, contact=None, description=None, location=None,
            os=None, vendor=None, sysobjectid=None):

        if not isinstance(Snmp, SnmpHandler):
            raise ValueError('Must pass a Nelmon SnmpHandler')

        self.contact = contact
        self.location = location
        self.sysobjectid = sysobjectid
        self.description = description
        self.os = os
        self.vendor = vendor
        self.version = None
        self.uptime = None
        self._snmp = Snmp

    def _parse_data(self, data):
        for oid, value in data:
            if o.sysContact in oid:
                self.contact = value
            if o.sysDescr in oid:
                self.description = value
            if o.sysLocation in oid:
                self.location = value
            if o.sysObjectId in oid:
                self.sysobjectid = value
            if o.sysUpTime in oid:
                self.uptime = value

    def get_all(self):
        oids = [
            o.sysContact + '.0',
            o.sysDescr + '.0',
            o.sysLocation + '.0',
            o.sysObjectId + '.0',
            o.sysUpTime + '.0',
        ]
        data = self._snmp.get(*oids)
        self._parse_data(data)
        self.get_vendor()
        self.get_version()

    def get_description(self):
        data = self._snmp.get(o.sysDescr + '.0')
        self._parse_data(data)

    def get_contact(self):
        data = self._snmp.get(o.sysContact + '.0')
        self._parse_data(data)

    def get_location(self):
        data = self._snmp.get(o.sysLocation + '.0')
        self._parse_data(data)

    def get_vendor(self):
        if self.sysobjectid is None:
            self.get_sysobjectid()
        try:
            enterprise_id = self.sysobjectid.split('.')[6]
            self.vendor = vendor_map[enterprise_id]
        except:
            self.vendor = 'UNKNOWN'

    def get_version(self):
        if self.vendor is None:
            self.get_vendor()
        if self.description is None:
            self.get_description()
        version_info = get_device_version(
            sysobjectid=self.sysobjectid,
            description=self.description,
            vendor=self.vendor,
            snmp=self._snmp
        )
        if self.vendor != version_info.vendor:
            self.vendor = version_info.vendor
        self.os = version_info.os
        self.version = version_info.version

    def get_sysobjectid(self):
        data = self._snmp.get(o.sysObjectId + '.0')
        self._parse_data(data)


class Hostinfo(HostInfo):
    '''Deprecated marked for removal, use HostInfo instead'''
    pass



from nelsnmp.hostinfo.version import DeviceVersion


class AristaVersion(DeviceVersion):

    def _get_version(self):
        for line in self._descriptions:
            if 'Arista Networks EOS' in line:
                self.os = 'eos'
                parts = line.split()
                if len(parts) > 4:
                    if parts[3] == 'version':
                        self.version = parts[4]



from nelsnmp.hostinfo.version import DeviceVersion
from nelsnmp.vendors.alcatel.versions import AlcatelVersion
from nelsnmp.vendors.arista.versions import AristaVersion
from nelsnmp.vendors.cisco.versions import CiscoVersion
from nelsnmp.vendors.ericsson.versions import EricssonVersion
from nelsnmp.vendors.huawei.versions import HuaweiVersion
from nelsnmp.vendors.juniper.versions import JuniperVersion
from nelsnmp.vendors.metamako.versions import MetamakoVersion
from nelsnmp.vendors.synology.versions import SynologyVersion
from nelsnmp.vendors.synology.oids import SynologyOids


def get_device_version(**kwargs):

    vendor = None
    for key in kwargs:
        if key == 'vendor':
            vendor = kwargs[key]

    if vendor == 'alcatel':
        return AlcatelVersion(**kwargs)
    if vendor == 'arista':
        return AristaVersion(**kwargs)
    if vendor == 'cisco':
        return CiscoVersion(**kwargs)
    if vendor == 'ericsson':
        return EricssonVersion(**kwargs)
    elif vendor == 'huawei':
        return HuaweiVersion(**kwargs)
    elif vendor == 'juniper':
        return JuniperVersion(**kwargs)
    elif vendor == 'metamako':
        return MetamakoVersion(**kwargs)
    elif vendor == 'net-snmp':
        if 'snmp' in kwargs.keys():
            found_vendor = get_netsnmp_device_vendor(kwargs['snmp'])
            if found_vendor:
                if found_vendor == 'synology':
                    kwargs['vendor'] = 'synology'
                    return SynologyVersion(**kwargs)
    return DeviceVersion(**kwargs)


def get_netsnmp_device_vendor(snmp):

    s = SynologyOids()
    vartable = snmp.getnext(s.systemStatus)
    for varbind in vartable:
        for oid, value in varbind:
            if s.systemStatus in oid:
                return 'synology'
    return None



from nelsnmp.hostinfo.version import DeviceVersion
from nelsnmp.vendors.airespace.oids import AirespaceOids


class CiscoVersion(DeviceVersion):

    def _get_version(self):
        internetwork_operating_system = False
        for line in self._descriptions:
            if 'IOS-XE Software' in line:
                self.os = 'ios-xe'
                parts = line.split(',')
                for part in parts:
                    if 'Version' in part:
                        version_strings = part.split()
                        if len(version_strings) > 1:
                            self.version = version_strings[1]
                break
            elif 'Cisco IOS Software' in line:
                self.os = 'ios'
                parts = line.split(',')
                for part in parts:
                    if 'Version' in part:
                        version_strings = part.split()
                        if len(version_strings) > 1:
                            self.version = version_strings[1]
                break
            elif 'Cisco Adaptive Security Appliance' in line:
                self.os = 'asa'
                parts = line.split()
                self.version = parts[-1]
            elif 'Cisco IOS XR Software' in line:
                self.os = 'ios-xr'
                parts = line.split(',')
                for part in parts:
                    if 'Version' in part:
                        version_strings = part.split()
                        if len(version_strings) > 1:
                            self.version = version_strings[1]
                break
            elif 'Cisco NX-OS' in line:
                self.os = 'nxos'
                parts = line.split(',')
                for part in parts:
                    if 'Version' in part:
                        version_strings = part.split()
                        if len(version_strings) > 1:
                            self.version = version_strings[1]
                            break
                break
            elif line == 'Cisco Controller':
                self._get_wlc_version()
            elif 'Cisco Internetwork Operating System Software' in line:
                self.os = 'ios'
                internetwork_operating_system = True
            if internetwork_operating_system:
                if 'IOS' in line:
                    parts = line.split(',')
                    if len(parts) > 1:
                        if 'Version' in parts[1]:
                            version_parts = parts[1].strip().split()
                            if len(version_parts) > 1:
                                self.version = version_parts[1]

    def _get_wlc_version(self):
        o = AirespaceOids()
        vartable = self._snmp.getnext(o.agentInventoryProductVersion)
        for varbinds in vartable:
            for oid, value in varbinds:
                if o.agentInventoryProductVersion in oid:
                    self.os = 'aireos'
                    self.version = value



from nelsnmp.snmp import SnmpHandler
from nelsnmp.vendors.cisco.oids import CiscoOids
o = CiscoOids()


class CdpNeighbors(object):
    def __init__(self, snmp=None):
        if not isinstance(snmp, SnmpHandler):
            raise ValueError('Must pass a Nelmon SnmpHandler')
        self._snmp = snmp
        self._raw_data = {}
        self.interface = {}

    def _get_interface_names(self):
        cdp_interfaces = []
        for interface in self._raw_data['neighbor_interfaces']:
            cdp_interfaces.append('%s.%s' % (o.ifDescr, interface))
        varbinds = self._snmp.get(*cdp_interfaces)
        for oid, value in varbinds:
            interface = oid.split('.')[-1]
            self.interface[interface] = value

    def _get_neighbors_data(self):
        vartable = self._snmp.getnext(o.cdpCacheEntry)
        neighbors = {}
        local_cdp_interfaces = []
        for varbind in vartable:
            for oid, value in varbind:
                entry = oid.rsplit('.', 2)[-1]
                interface = oid.rsplit('.', 2)[-2]
                if entry not in neighbors.keys():
                    neighbors[entry] = {}
                if interface not in local_cdp_interfaces:
                    local_cdp_interfaces.append(interface)
                if interface not in neighbors[entry].keys():
                    neighbors[entry][interface] = {}
                if o.cdpCacheDeviceId in oid:
                    neighbors[entry][interface]['cdpCacheDeviceId'] = value
                if o.cdpCacheDevicePort in oid:
                    neighbors[entry][interface]['cdpCacheDevicePort'] = value
        self._raw_data['neighbors'] = neighbors
        self._raw_data['neighbor_interfaces'] = local_cdp_interfaces
        self._get_interface_names()

    def get_cdp_neighbors_dict(self):
        self._get_neighbors_data()
        cdp = {}
        neighbors = self._raw_data['neighbors']
        for entry in neighbors:
            for interface in neighbors[entry]:
                if_name = self.interface[interface]
                neighbor_host = neighbors[entry][interface]['cdpCacheDeviceId']
                if if_name not in cdp.keys():
                    cdp[if_name] = {}
                if neighbor_host not in cdp[if_name].keys():
                    cdp[if_name][neighbor_host] = {}
                cdp[if_name][neighbor_host]['interface'] = neighbors[entry][interface]['cdpCacheDevicePort']
        self.neighbors = cdp

    def get_cdp_neighbors_list(self):
        self._get_neighbors_data()
        cdp = {}
        neighbors = self._raw_data['neighbors']
        for entry in neighbors:
            for interface in neighbors[entry]:
                if_name = self.interface[interface]
                neighbor_host = neighbors[entry][interface]['cdpCacheDeviceId']
                if if_name not in cdp.keys():
                    cdp[if_name] = []
                data = {}
                data['neighbor'] = neighbor_host
                data['interface'] = neighbors[entry][interface]['cdpCacheDevicePort']
                cdp[if_name].append(data)
        self.neighbors = cdp



import re


from codecs import open
from setuptools import setup, find_packages

version = ''
with open('lib/nelsnmp/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()

with open('HISTORY.rst', 'r', 'utf-8') as f:
    history = f.read()

long_description = readme + '\n\n' + history

config = {
    'name': 'nelsnmp',
    'package_dir': {'': 'lib'},
    'packages': find_packages('lib'),
    'version': version,
    'description': 'A wrapper module for pysnmp',
    'long_description': long_description,
    'author': 'Patrick Ogenstad',
    'author_email': 'patrick@ogenstad.com',
    'license': 'Apache',
    'url': 'https://networklore.com/nelsnmp/',
    'install_requires': ['pysnmp >= 4.3.1'],
    'classifiers': ['Development Status :: 4 - Beta',
                    'Intended Audience :: Developers',
                    'Intended Audience :: System Administrators']
}

setup(**config)
